@@ -30,7 +30,7 @@ _svmf_cast_$1
 inline static $2
 _svmf_cast_$1 ($3 expr)
 {
-  return ($2) expr;
+  return ($2) (void *) expr;
 }
 
 m4svm_define_end v = ":])";
