@@ -12,10 +12,9 @@ _svmf_resolve_super_interfaces
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_resolve_super_interfaces (_svmt_JNIEnv *env, _svmt_class_info *class)
 {
-  _svmt_JavaVM *vm = env->vm;
   jint i;
 
   assert (class->class_loader_info->class_loader != NULL);
@@ -59,7 +58,7 @@ _svmf_resolve_super_class
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_resolve_super_class (_svmt_JNIEnv *env, _svmt_class_info *class)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -113,7 +112,7 @@ _svmf_bootcl_resolve_super_interfaces
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_bootcl_resolve_super_interfaces (_svmt_JNIEnv *env,
 				       _svmt_class_info *class)
 {
@@ -175,7 +174,7 @@ _svmf_bootcl_resolve_super_class
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_bootcl_resolve_super_class (_svmt_JNIEnv *env, _svmt_class_info *class)
 {
   _svmt_JavaVM *vm = env->vm;
@@ -255,7 +254,7 @@ _svmf_bootcl_derive_class
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_bootcl_derive_class (_svmt_JNIEnv *env,
 			   const char *class_name,
 			   _svmt_class_file *class_file,
@@ -288,7 +287,7 @@ _svmf_bootcl_derive_class (_svmt_JNIEnv *env,
     }
 
   /* were're done with the bytes, so free them to allow recursive calls */
-  if (free_bytes)
+  if (free_bytes && (class_file->free_bytes_after_use == JNI_TRUE))
     {
       _svmm_gmfree_ubytes (class_file->bytes);
       class_file->length = 0;
@@ -353,7 +352,9 @@ _svmf_bootcl_derive_class (_svmt_JNIEnv *env,
 
   if (vm->class_loading.boot_loader.classes.jlclass != NULL)
     {
-      if (_svmf_new_class (env, _svmf_cast_type_class (class)) != JNI_OK)
+      if (_svmf_new_class_pd
+	  (env, _svmf_cast_type_class (class),
+	   class->protection_domain) != JNI_OK)
 	{
 	  _svmt_type_node type = { NULL, NULL, NULL, NULL, NULL };
 	  _svmt_type_node *partially_derived_type;
@@ -398,26 +399,71 @@ _svmf_bootcl_derive_class (_svmt_JNIEnv *env,
   return JNI_OK;
 }
 
+
 /*
 ----------------------------------------------------------------------
-_svmh_bootcl_internal_load_class_file
+_svmf_bootcl_internal_load_zipped_class_file
 ----------------------------------------------------------------------
 */
 
-static jint
-_svmh_bootcl_internal_load_class_file (_svmt_JNIEnv *env,
-				       const char *class_name,
-				       _svmt_class_file *class_file)
+svm_static jint
+  _svmf_bootcl_internal_load_zipped_class_file
+  (_svmt_JNIEnv *env, _svmt_zip_file ** zip_file, const char *class_path,
+   const char *class_name, _svmt_class_file *class_file,
+   jboolean *free_after_use)
+{
+  jint result;
+  size_t class_file_name_length;
+  char *class_file_name;
+
+
+  assert (zip_file != NULL);
+
+  if (*zip_file == NULL)
+    {
+      *zip_file = _svmf_zip_file_open (env, class_path);
+      if (*zip_file == NULL)
+	return JNI_ERR;
+    }
+
+  class_file_name_length = strlen (class_name) + sizeof (".class");
+
+  if (_svmm_gmalloc_cchars (env, strlen (class_name) + sizeof (".class"),
+			    class_file_name) != JNI_OK)
+    return JNI_ERR;
+
+  strcpy (class_file_name, class_name);
+  strcat (class_file_name, ".class");
+
+  result = _svmf_zip_get_file_content
+    (env, &(class_file->bytes), &(class_file->length), *zip_file,
+     class_file_name, free_after_use);
+
+  if (result == ZIP_FILE_FOUND)
+    return JNI_OK;
+
+  return JNI_ERR;
+}
+
+/*
+----------------------------------------------------------------------
+_svmf_bootcl_internal_load_ondisk_class_file
+----------------------------------------------------------------------
+*/
+
+svm_static jint
+  _svmf_bootcl_internal_load_ondisk_class_file
+  (_svmt_JNIEnv *env, const char *class_path,
+   const char *class_name, _svmt_class_file *class_file)
 {
-  _svmt_JavaVM *vm = env->vm;
   int fd;
   struct stat stats;
   void *content;
 
   {
     size_t path_length =
-      strlen (vm->class_loading.boot_loader.boot_class_path) + 1 +
-      strlen (class_name) + 6 + 1;
+      strlen (class_path) + sizeof ('/') + strlen (class_name) +
+      strlen (".class");
     char *path;
 
     path = _svmf_malloc (path_length);
@@ -428,7 +474,7 @@ _svmh_bootcl_internal_load_class_file (_svmt_JNIEnv *env,
 	return JNI_ERR;
       }
 
-    strcpy (path, vm->class_loading.boot_loader.boot_class_path);
+    strcpy (path, class_path);
 
     if (path[strlen (path) - 1] != '/')
       {
@@ -441,7 +487,6 @@ _svmh_bootcl_internal_load_class_file (_svmt_JNIEnv *env,
     if ((fd = open (path, O_RDONLY)) == -1)
       {
 	_svmf_free (path);
-	_svmf_error_NoClassDefFoundError (env);
 	return JNI_ERR;
       }
 
@@ -452,7 +497,6 @@ _svmh_bootcl_internal_load_class_file (_svmt_JNIEnv *env,
   if (fstat (fd, &stats) == -1)
     {
       close (fd);
-      _svmf_error_NoClassDefFoundError (env);
       return JNI_ERR;
     }
 
@@ -464,6 +508,7 @@ _svmh_bootcl_internal_load_class_file (_svmt_JNIEnv *env,
       class_file->length = 0;
       close (fd);
       _svmf_error_ClassFormatError (env);
+
       return JNI_ERR;
     }
 
@@ -474,6 +519,7 @@ _svmh_bootcl_internal_load_class_file (_svmt_JNIEnv *env,
       class_file->length = 0;
       close (fd);
       _svmf_error_OutOfMemoryError (env);
+
       return JNI_ERR;
     }
 
@@ -483,7 +529,7 @@ _svmh_bootcl_internal_load_class_file (_svmt_JNIEnv *env,
       munmap (content, class_file->length);
       class_file->length = 0;
       close (fd);
-      _svmf_error_OutOfMemoryError (env);
+
       return JNI_ERR;
     }
 
@@ -495,17 +541,369 @@ _svmh_bootcl_internal_load_class_file (_svmt_JNIEnv *env,
   return JNI_OK;
 }
 
+/*
+----------------------------------------------------------------------
+_svmf_bootcl_internal_split_bootcl
+----------------------------------------------------------------------
+*/
+
+svm_static jint
+_svmf_bootcl_internal_split_bootcl (_svmt_JNIEnv *env)
+{
+  _svmt_JavaVM *vm = env->vm;
+
+  /*
+   * boot.class.path = boot.class.path.prepend + ':'
+   *  + boot.class.path + ':' + boot.class.path.append
+   */
+
+  {
+    jint bootcp_length = 0;
+
+    if (vm->class_loading.boot_loader.boot_class_path_prepend != NULL)
+      bootcp_length += strlen
+	(vm->class_loading.boot_loader.boot_class_path_prepend);
+
+    if (vm->class_loading.boot_loader.boot_class_path_append != NULL)
+      bootcp_length += strlen
+	(vm->class_loading.boot_loader.boot_class_path_append);
+
+    if (bootcp_length > 0)
+      {
+
+	char *new_boot_class_path = NULL;
+
+	if (vm->class_loading.boot_loader.boot_class_path != NULL)
+	  bootcp_length += strlen
+	    (vm->class_loading.boot_loader.boot_class_path) + 1;
+
+	bootcp_length += 3;
+
+	if (_svmm_gzmalloc_cchars_no_exception
+	    (bootcp_length, new_boot_class_path) != JNI_OK)
+	  return JNI_ERR;
+
+	if (vm->class_loading.boot_loader.boot_class_path_prepend != NULL)
+	  {
+	    strcat (new_boot_class_path,
+		    vm->class_loading.boot_loader.boot_class_path_prepend);
+	    strcat (new_boot_class_path, ":");
+	  }
+
+	if (vm->class_loading.boot_loader.boot_class_path != NULL)
+	  strcat (new_boot_class_path,
+		  vm->class_loading.boot_loader.boot_class_path);
+
+	if (vm->class_loading.boot_loader.boot_class_path_append != NULL)
+	  {
+	    strcat (new_boot_class_path, ":");
+	    strcat (new_boot_class_path,
+		    vm->class_loading.boot_loader.boot_class_path_append);
+	  }
+
+	_svmm_gzmfree_cchars_no_exception
+	  (vm->class_loading.boot_loader.boot_class_path);
+	vm->class_loading.boot_loader.boot_class_path = new_boot_class_path;
+      }
+
+    if (vm->class_loading.boot_loader.boot_class_path_prepend != NULL)
+      _svmm_gzmfree_cchars_no_exception
+	(vm->class_loading.boot_loader.boot_class_path_prepend);
+
+    if (vm->class_loading.boot_loader.boot_class_path_append != NULL)
+      _svmm_gzmfree_cchars_no_exception
+	(vm->class_loading.boot_loader.boot_class_path_append);
+  }
+
+  /* Actually split full boot class path. */
+  {
+    jint entries_no = 0;
+    char *colon_pos, *position;
+
+    position = vm->class_loading.boot_loader.boot_class_path;
+
+    /* count the entries */
+    while (*position != 0)
+      {
+	entries_no++;
+	/* Skip multiple ":" */
+	while (*position == ':')
+	  position++;
+
+	colon_pos = strchr (position, ':');
+
+	if (colon_pos == NULL)
+	  break;
+
+	position = ++colon_pos;
+      }
+
+    /* allocate needed amount of entries */
+    if (JNI_OK != _svmm_gzmalloc_boot_class_path_entry
+	(env, entries_no,
+	 vm->class_loading.boot_loader.boot_class_path_entries))
+      return JNI_ERR;
+
+    /* fill in entries' data */
+    {
+      _svmt_boot_class_path_entry *entries =
+	vm->class_loading.boot_loader.boot_class_path_entries;
+
+      entries_no = 0;
+      position = vm->class_loading.boot_loader.boot_class_path;
+
+      while (*position != 0)
+	{
+	  jint len;
+	  struct stat file_stat;
+
+	  entries_no++;
+
+	  /* Skip multiple ":" */
+	  while (*position == ':')
+	    position++;
+
+	  colon_pos = strchr (position, ':');
+
+	  if (colon_pos == NULL)
+	    len = strlen (position);
+	  else
+	    len = (jint) (colon_pos - position);
+
+	  if (JNI_OK != _svmm_gmalloc_cchars
+	      (env, len + 1, entries[entries_no - 1].class_path))
+	    {
+	      _svmf_error_OutOfMemoryError (env);
+	      return JNI_ERR;
+	    }
+
+	  memcpy (entries[entries_no - 1].class_path, position, len);
+
+	  entries[entries_no - 1].class_path[len] = 0;
+
+	  /* record whether it is a file or directory */
+	  if (stat (entries[entries_no - 1].class_path, &file_stat) != 0)
+	    {
+	      entries[entries_no - 1].is_valid = JNI_FALSE;
+	    }
+	  else
+	    {
+	      entries[entries_no - 1].is_valid = JNI_TRUE;
+	      if (S_ISDIR (file_stat.st_mode))
+		entries[entries_no - 1].is_file = JNI_FALSE;
+	      else
+		entries[entries_no - 1].is_file = JNI_TRUE;
+	    }
+
+	  entries[entries_no - 1].zip_file = NULL;
+
+	  vm->class_loading.boot_loader.boot_class_path_entries_no =
+	    entries_no;
+
+	  if (colon_pos == NULL)
+	    break;
+
+	  position = ++colon_pos;
+	}
+    }
+  }
+
+  return JNI_OK;
+}
+
+/*
+----------------------------------------------------------------------
+_svmh_bootcl_internal_load_class_file
+----------------------------------------------------------------------
+*/
+
+svm_static jint
+_svmh_bootcl_internal_load_class_file (_svmt_JNIEnv *env,
+				       const char *class_name,
+				       _svmt_class_file *class_file)
+{
+  _svmt_JavaVM *vm = env->vm;
+
+  /*
+   * Traverse boot_class_path_elements try to load the class data.
+   *
+   * If JNI_ERR was returned then:
+   * - *(env->throwable) == NULL means ClassNotFound,
+   * - *(env->throwable) != NULL means a serious error had place (like
+   *   memory allocation error).
+   */
+  {
+    jint entry_no;
+    for (entry_no = 0;
+	 entry_no < vm->class_loading.boot_loader.boot_class_path_entries_no;
+	 entry_no++)
+      {
+	_svmt_boot_class_path_entry *entry =
+	  &vm->class_loading.boot_loader.boot_class_path_entries[entry_no];
+
+	if (entry->is_valid == JNI_FALSE)
+	  continue;
+
+	if (entry->is_file == JNI_FALSE)
+	  {			/* directory - look for an uncompressed .class file */
+	    if (JNI_OK == _svmf_bootcl_internal_load_ondisk_class_file
+		(env, entry->class_path, class_name, class_file))
+	      {
+		class_file->free_bytes_after_use = JNI_TRUE;
+		break;		/* class found! */
+	      }
+	  }
+	else
+	  {			/* file - means .jar or .zip usually */
+	    if (JNI_OK == _svmf_bootcl_internal_load_zipped_class_file
+		(env, &(entry->zip_file), entry->class_path, class_name,
+		 class_file, &(class_file->free_bytes_after_use)))
+	      break;		/* class found! */
+	  }
+      }
+
+    if (*(env->throwable) != NULL)
+      return JNI_ERR;
+
+    if (entry_no == vm->class_loading.boot_loader.boot_class_path_entries_no)
+      {
+	/* At bootstrap time  _svmf_error_NoClassDefFoundError never returns.
+	   Detect it and try to provide some meaningful message to user. */
+	if (vm->initialization != NULL)
+	  {
+	    _svmf_printf (env, stderr,
+			  "*** Fatal error: Unable to load essential class %s.\n",
+			  class_name);
+	    _svmf_printf (env, stderr,
+			  "*** Most probably it was not found on boot classpath:\n");
+	    _svmf_printf (env, stderr, "*** %s\n",
+			  vm->class_loading.boot_loader.boot_class_path);
+	  }
+
+	_svmf_error_NoClassDefFoundError (env);
+
+	return JNI_ERR;
+      }
+  }
+
+  return JNI_OK;
+}
+
+/*
+----------------------------------------------------------------------
+_svmf_bootcl_get_resource_url
+----------------------------------------------------------------------
+*/
+
+svm_static char *
+_svmf_bootcl_get_resource_url (_svmt_JNIEnv *env, const char *res_name)
+{
+  _svmt_JavaVM *vm = env->vm;
+
+  {
+    jint entry_no;
+
+    if (res_name[0] == '/')
+      res_name++;
+
+    for (entry_no = 0;
+	 entry_no < vm->class_loading.boot_loader.boot_class_path_entries_no;
+	 entry_no++)
+      {
+	_svmt_boot_class_path_entry *entry =
+	  &vm->class_loading.boot_loader.boot_class_path_entries[entry_no];
+
+	if (entry->is_valid == JNI_FALSE)
+	  continue;
+
+	if (entry->is_file == JNI_FALSE)
+	  {			/* directory - look for an uncompressed .class file */
+
+	    struct stat stats;
+	    size_t path_length = sizeof ("file://") +
+	      strlen (entry->class_path) + sizeof ('/') + strlen (res_name);
+	    char *path;
+
+	    path = _svmf_malloc (path_length);
+
+	    if (path == NULL)
+	      {
+		_svmf_error_OutOfMemoryError (env);
+		return NULL;
+	      }
+
+	    strcpy (path, "file://");
+	    strcat (path, entry->class_path);
+
+	    if (path[strlen (path) - 1] != '/')
+	      {
+		strcat (path, "/");
+	      }
+
+	    strcat (path, res_name);
+
+	    if (stat (path + sizeof ("file://") - 1, &stats) == 0)
+	      {			/* we found the file! */
+		return path;
+	      }
+	  }
+	else
+	  {			/* file - means .jar or .zip usually */
+	    if (entry->zip_file == NULL)
+	      {
+		entry->zip_file =
+		  _svmf_zip_file_open (env, entry->class_path);
+		if (entry->zip_file == NULL)
+		  continue;
+	      }
+
+	    if (_svmf_zip_file_exists (env, entry->zip_file, res_name) ==
+		JNI_TRUE)
+	      {
+
+		size_t path_length = sizeof ("jar:file://") +
+		  strlen (entry->class_path) + sizeof ("!/") +
+		  strlen (res_name);
+		char *path;
+
+		path = _svmf_malloc (path_length);
+
+		if (path == NULL)
+		  {
+		    _svmf_error_OutOfMemoryError (env);
+		    return NULL;
+		  }
+
+		strcpy (path, "jar:file://");
+		strcat (path, entry->class_path);
+		strcat (path, "!/");
+		strcat (path, res_name);
+
+		return path;
+
+	      }
+	  }
+
+      }
+  }
+
+  return NULL;
+}
+
 /*
 ----------------------------------------------------------------------
 _svmf_bootcl_create_class
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_bootcl_create_class (_svmt_JNIEnv *env,
 			   const char *class_name, _svmt_class_info **pclass)
 {
   _svmt_JavaVM *vm = env->vm;
+  _svmt_class_file current_class_file;
+
+  _svmm_zero_memory (current_class_file);
 
   /* has the boot loader been recorded as an initiating loader of "name"? */
   {
@@ -526,38 +924,36 @@ _svmf_bootcl_create_class (_svmt_JNIEnv *env,
 
   if (vm->verbose_class)
     {
-      _svmf_printf (env, stdout, "[verbose class: loading \"%s\"]\n",
+      _svmf_printf (env, stderr, "[verbose class: loading \"%s\"]\n",
 		    class_name);
     }
 
-  assert (vm->class_loading.boot_loader.current_class_file.bytes == NULL);
+  assert (current_class_file.bytes == NULL);
 
   if (_svmm_bootcl_internal_load_class_file
-      (env, class_name,
-       vm->class_loading.boot_loader.current_class_file) != JNI_OK)
+      (env, class_name, current_class_file) != JNI_OK)
     {
       return JNI_ERR;
     }
 
   if (_svmf_bootcl_derive_class
-      (env, class_name, &vm->class_loading.boot_loader.current_class_file,
-       pclass, JNI_TRUE) != JNI_OK)
+      (env, class_name, &current_class_file, pclass, JNI_TRUE) != JNI_OK)
     {
-      if (vm->class_loading.boot_loader.current_class_file.bytes != NULL)
+      if ((current_class_file.bytes != NULL) &&
+	  (current_class_file.free_bytes_after_use == JNI_TRUE))
 	{
-	  _svmm_gmfree_ubytes (vm->class_loading.boot_loader.
-			       current_class_file.bytes);
-	  vm->class_loading.boot_loader.current_class_file.length = 0;
+	  _svmm_gmfree_ubytes (current_class_file.bytes);
+	  current_class_file.length = 0;
 	}
 
       return JNI_ERR;
     }
 
-  if (vm->class_loading.boot_loader.current_class_file.bytes != NULL)
+  if ((current_class_file.bytes != NULL) &&
+      (current_class_file.free_bytes_after_use == JNI_TRUE))
     {
-      _svmm_gmfree_ubytes (vm->class_loading.boot_loader.current_class_file.
-			   bytes);
-      vm->class_loading.boot_loader.current_class_file.length = 0;
+      _svmm_gmfree_ubytes (current_class_file.bytes);
+      current_class_file.length = 0;
     }
 
   return JNI_OK;
@@ -569,7 +965,7 @@ _svmf_usercl_create_class
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_usercl_create_class (_svmt_JNIEnv *env,
 			   _svmt_class_loader_info *class_loader_info,
 			   const char *class_name, _svmt_class_info **pclass)
@@ -620,7 +1016,7 @@ _svmh_create_class
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmh_create_class (_svmt_JNIEnv *env,
 		    _svmt_class_loader_info *class_loader_info,
 		    const char *class_name, _svmt_class_info **pclass)
@@ -630,10 +1026,7 @@ _svmh_create_class (_svmt_JNIEnv *env,
 
   if (vm->initialization == NULL)
     {
-      if (_svmf_enter_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+      if (_svmf_enter_class_loader_monitor (env, class_loader_info) != JNI_OK)
 	{
 	  goto error;
 	}
@@ -668,10 +1061,7 @@ _svmh_create_class (_svmt_JNIEnv *env,
     {
       monitor_acquired = JNI_FALSE;
 
-      if (_svmf_exit_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+      if (_svmf_exit_class_loader_monitor (env, class_loader_info) != JNI_OK)
 	{
 	  goto error;
 	}
@@ -685,10 +1075,7 @@ error:
     {
       monitor_acquired = JNI_FALSE;
 
-      if (_svmf_exit_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+      if (_svmf_exit_class_loader_monitor (env, class_loader_info) != JNI_OK)
 	{
 	  goto error;
 	}
@@ -703,7 +1090,7 @@ _svmf_bootcl_create_array
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_bootcl_create_array (_svmt_JNIEnv *env,
 			   const char *array_name, _svmt_array_info **parray)
 {
@@ -729,7 +1116,7 @@ _svmf_bootcl_create_array (_svmt_JNIEnv *env,
 
   if (vm->verbose_class)
     {
-      _svmf_printf (env, stdout, "[verbose class: creating \"%s\"]\n",
+      _svmf_printf (env, stderr, "[verbose class: creating \"%s\"]\n",
 		    array_name);
     }
 
@@ -912,7 +1299,7 @@ _svmf_usercl_create_array
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_usercl_create_array (_svmt_JNIEnv *env,
 			   _svmt_class_loader_info *class_loader_info,
 			   const char *array_name, _svmt_array_info **parray)
@@ -963,7 +1350,7 @@ _svmh_create_array
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmh_create_array (_svmt_JNIEnv *env,
 		    _svmt_class_loader_info *class_loader_info,
 		    const char *array_name, _svmt_array_info **parray)
@@ -973,10 +1360,7 @@ _svmh_create_array (_svmt_JNIEnv *env,
 
   if (vm->initialization == NULL)
     {
-      if (_svmf_enter_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+      if (_svmf_enter_class_loader_monitor (env, class_loader_info) != JNI_OK)
 	{
 	  goto error;
 	}
@@ -1011,10 +1395,7 @@ _svmh_create_array (_svmt_JNIEnv *env,
     {
       monitor_acquired = JNI_FALSE;
 
-      if (_svmf_exit_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+      if (_svmf_exit_class_loader_monitor (env, class_loader_info) != JNI_OK)
 	{
 	  goto error;
 	}
@@ -1028,10 +1409,7 @@ error:
     {
       monitor_acquired = JNI_FALSE;
 
-      if (_svmf_exit_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+      if (_svmf_exit_class_loader_monitor (env, class_loader_info) != JNI_OK)
 	{
 	  goto error;
 	}
@@ -1046,7 +1424,7 @@ _svmh_create_type
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmh_create_type (_svmt_JNIEnv *env,
 		   _svmt_class_loader_info *class_loader_info,
 		   const char *type_name, _svmt_type_info **ptype)
