@@ -21,14 +21,15 @@ _svmh_create_array (env, cl, name, &array)
 #define _svmm_bootcl_internal_load_class_file(env,name,cf) \
 _svmh_bootcl_internal_load_class_file (env, name, &cf)
 
-static jint _svmh_create_type (_svmt_JNIEnv *env,
-			       _svmt_class_loader_info *class_loader_info,
-			       const char *type_name,
-			       _svmt_type_info **ptype);
+svm_static jint _svmh_create_type (_svmt_JNIEnv *env,
+				   _svmt_class_loader_info *class_loader_info,
+				   const char *type_name,
+				   _svmt_type_info **ptype);
 
-static jint _svmh_create_array (_svmt_JNIEnv *env,
-				_svmt_class_loader_info *class_loader_info,
-				const char *array_name,
-				_svmt_array_info **parray);
+svm_static jint _svmh_create_array (_svmt_JNIEnv *env,
+				    _svmt_class_loader_info
+				    *class_loader_info,
+				    const char *array_name,
+				    _svmt_array_info **parray);
 
 #endif /* SVM_CLASS_LOADER_H */
