@@ -102,9 +102,7 @@
 #define SVM_THREAD_INTERRUPTIBLE_ON_FAT_LOCK 8
 
 /* needed to avoid explicit m4 in instructions.m4.c */
-#ifdef SABLEVM_GC_WRITE_BARRIER
-#define _SABLEVM_GC_WRITE_BARRIER_REFERENCE
-#endif
+#define SVM_PUT_REFERENCE_FIELD 1
 
 /* min/max values of java integer types */
 #define SVM_JBYTE_MIN (-128)
