@@ -138,6 +138,7 @@ _svmf_internal_sigaction (int signo, siginfo_t * info, void *ignored)
 
     case SIGSEGV:
       {
+	jint status = env->thread_status;
 
 #ifndef NDEBUG
 
@@ -153,7 +154,28 @@ _svmf_internal_sigaction (int signo, siginfo_t * info, void *ignored)
 
 #endif /* not NDEBUG */
 
-	_svmf_signal_throw (env, SVM_SIGNAL_NULL_POINTER_EXCEPTION);
+	if (status == SVM_THREAD_STATUS_NOT_RUNNING_JAVA_RESUMING_ALLOWED ||
+	    status == SVM_THREAD_STATUS_NOT_RUNNING_JAVA_RESUMING_DISALLOWED)
+	  {
+	    volatile _svmt_word *cause_segfault = NULL;
+	    _svmf_printf (env, stderr,
+			  "** Segfault in native code on thread %d.  Java stack trace follows.\n",
+			  env->thread.id);
+	    _svmf_dump_stack_trace (env);
+	    fflush (NULL);
+	    /* Stack trace can be long, let the user know (again) what happened. */
+	    _svmf_printf (env, stderr,
+			  "** Segfault in native code on thread %d.  Java stack trace done.\n",
+			  env->thread.id);
+	    /* We force a "real" segfault to get core dump and ease debugging. */
+	    *cause_segfault = 0xdeadbeef;
+	    _svmm_fatal_error
+	      ("Impossible control flow after segmentation fault catch.");
+	  }
+	else
+	  {
+	    _svmf_signal_throw (env, SVM_SIGNAL_NULL_POINTER_EXCEPTION);
+	  }
 	goto delegate;
       }
       break;
