@@ -9,19 +9,14 @@
 #ifndef SVM_FATAL_H
 #define SVM_FATAL_H
 
-/* The __func__ is only defined in C99, so we can't rely on it being defined.
-   The __FUNCTION__ OTOH is a non-standard define available in older GCC. */
-#ifdef __func__
+/* The __func__ is only defined in C99, and it CANNOT be tested by #ifdef,
+   so we use autoconf test that sets HAVE___func__ for us. */
+#ifdef HAVE___func__
 #  define _svmm_fatal_error(msg) \
      _svmh_fatal_error(__FILE__, __func__, __LINE__, msg)
 #else
-#  ifdef __FUNCTION__
-#    define _svmm_fatal_error(msg) \
-       _svmh_fatal_error(__FILE__, __FUNCTION__, __LINE__, msg)
-#  else
-#    define _svmm_fatal_error(msg) \
-       _svmh_fatal_error(__FILE__, "unknown", __LINE__, msg)
-#  endif
+#  define _svmm_fatal_error(msg) \
+     _svmh_fatal_error(__FILE__, "unknown", __LINE__, msg)
 #endif
 
 #endif /* not SVM_FATAL_H */
