@@ -1,6 +1,8 @@
 #ifndef GC_GENCOPY_H
 #define GC_GENCOPY_H
 
+#if defined (_SABLEVM_GENCOPY_GC)
+
 #define _SABLEVM_GC_WRITE_BARRIER 1
 
 svm_static void _svmf_remember_set_add (_svmt_remember_set * remset,
@@ -39,4 +41,6 @@ _svmf_write_barrier (_svmt_JNIEnv *env, _svmt_object_instance **ref)
     }
 }
 
+#endif /* _SABLEVM_GENCOPY_GC */
+
 #endif /* GC_GENCOPY_H */
