@@ -507,18 +507,18 @@ Java_gnu_classpath_VMSystemProperties_getBootstrapClassPath (JNIEnv *_env,
 
 /*
 ----------------------------------------------------------------------
-Java_gnu_classpath_VMSystemProperties_getExtDirsPath
+Java_gnu_classpath_VMSystemProperties_getJavaHome
 ----------------------------------------------------------------------
 */
 
 /*
  * Class:     gnu_classpath_VMSystemProperties
- * Method:    getExtDirsPath
+ * Method:    getJavaHome
  * Signature: ()Ljava/lang/String;
  */
 JNIEXPORT static jstring JNICALL
-Java_gnu_classpath_VMSystemProperties_getExtDirsPath (JNIEnv *_env,
-						      jclass class SVM_UNUSED)
+Java_gnu_classpath_VMSystemProperties_getJavaHome (JNIEnv *_env,
+						   jclass class SVM_UNUSED)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
   jstring string;
@@ -529,7 +529,7 @@ Java_gnu_classpath_VMSystemProperties_getExtDirsPath (JNIEnv *_env,
     /* as we do nothing else after, we can safely ignore the return
        value (any exception being also set in the environment) */
     string = _svmf_get_jni_frame_native_local (env);
-    _svmf_get_string (env, JAVA_HOME_PATH "/jre/lib/ext", string);
+    _svmf_get_string (env, JAVA_HOME_PATH, string);
   }
 
   _svmm_stopping_java (env);
