@@ -480,7 +480,7 @@ struct JNINativeInterface_struct
 				       const jchar *chars);	/* 166 */
   jstring (JNICALL *NewStringUTF) (JNIEnv *env, const char *bytes);	/* 167 */
   jsize (JNICALL *GetStringUTFLength) (JNIEnv *env, jstring string);	/* 168 */
-  const jbyte *(JNICALL *GetStringUTFChars) (JNIEnv *env, jstring string,
+  const char *(JNICALL *GetStringUTFChars) (JNIEnv *env, jstring string,
 					      jboolean *isCopy);	/* 169 */
   void (JNICALL *ReleaseStringUTFChars) (JNIEnv *env, jstring string,
 					  const char *utf);	/* 170 */
