@@ -201,11 +201,11 @@ $1
 	{
 #ifndef	NDEBUG
 
-	  if (env->vm->verbose_instructions)
+	  if (env->vm->verbose_instructions == JNI_TRUE)
 	    {
 	      _svmf_printf (env, stderr,
-			    "[verbose instructions: executing @%p $1]\n",
-			    (void *) (pc - 1));
+			    "T%d: [verbose instructions: executing @%p $1]\n",
+			    env->thread.id, (void *) (pc - 1));
 	    }
 
 #endif
@@ -269,11 +269,11 @@ $1
       START_$1:
 #ifndef	NDEBUG
 
-	if (env->vm->verbose_instructions)
+	if (env->vm->verbose_instructions == JNI_TRUE)
 	  {
 	    _svmf_printf (env, stderr,
-			  "[verbose instructions: executing @%p $1]\n",
-			  (void *) (pc - 1));
+			  "T%d: [verbose instructions: executing @%p $1]\n",
+			  env->thread.id, (void *) (pc - 1));
 	  }
 
 #endif
