@@ -25,7 +25,7 @@ Java_java_io_VMObjectStreamClass_hasClassInitializer (JNIEnv *_env,
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
   jboolean result;
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     _svmt_type_info *type = _svmf_unwrap_class_instance (env, clazz);
@@ -54,7 +54,7 @@ Java_java_io_VMObjectStreamClass_hasClassInitializer (JNIEnv *_env,
       }
   }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 
   return result;
 }
