@@ -18,11 +18,11 @@ Java_java_lang_VMThread_yield
  * Signature: ()V
  */
 
-static JNIEXPORT void JNICALL
+JNIEXPORT static void JNICALL
 Java_java_lang_VMThread_yield (JNIEnv *_env, jclass _class SVM_UNUSED)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   if (sched_yield () != 0)
     {
@@ -35,11 +35,89 @@ Java_java_lang_VMThread_yield (JNIEnv *_env, jclass _class SVM_UNUSED)
     }
 
 end:
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 
   return;
 }
 
+/*
+----------------------------------------------------------------------
+Java_java_lang_VMThread_sleep
+----------------------------------------------------------------------
+*/
+
+/*
+ * Class:     java_lang_VMThread
+ * Method:    sleep
+ * Signature: ()Z
+ */
+JNIEXPORT static void JNICALL
+Java_java_lang_VMThread_sleep (JNIEnv *_env, jclass this SVM_UNUSED, jlong ms,
+			       jint ns)
+{
+
+  _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
+  struct timespec timeout;
+  _svmt_word old_interrupted_status, new_interrupted_status;
+
+  do
+    {
+      old_interrupted_status = env->thread.interrupted_status;
+      new_interrupted_status = old_interrupted_status;
+
+      new_interrupted_status |= SVM_THREAD_INTERRUPTIBLE_BY_SIGNAL;
+      new_interrupted_status &= ~SVM_THREAD_THROW_INTERRUPTED;
+    }
+  while (!_svmm_compare_and_swap
+	 (env->thread.interrupted_status, old_interrupted_status,
+	  new_interrupted_status));
+
+  timeout.tv_sec = ms / 1000;
+  timeout.tv_nsec = (ms % 1000) * 1000 * 1000 + ns;
+
+  /* Watch for overflow of timeout.tv_nsec...
+     OK; I assume that tv_nsec has at least 32 bits (signed or
+     not, I don't care), a reasonable assumption given that it
+     must be able to hold the value 999999999.
+   */
+  if (timeout.tv_nsec >= 1000000000L)
+    {
+      timeout.tv_sec++;
+      timeout.tv_nsec -= 1000000000L;
+    }
+
+  /* we don't check the return value, as in case of interrupt() that
+   * caused a signal that woke us up, we'll have interrupted_status
+   * flag set accordingly */
+  nanosleep (&timeout, NULL);
+
+  do
+    {
+      old_interrupted_status = env->thread.interrupted_status;
+      new_interrupted_status = old_interrupted_status;
+
+      if (old_interrupted_status & SVM_THREAD_THROW_INTERRUPTED)
+	{
+	  new_interrupted_status &= ~SVM_THREAD_INTERRUPTIBLE_ON_FAT_LOCK;
+	  new_interrupted_status &= ~SVM_THREAD_THROW_INTERRUPTED;
+	}
+      else
+	{
+	  break;
+	}
+    }
+  while (!_svmm_compare_and_swap
+	 (env->thread.interrupted_status,
+	  old_interrupted_status, new_interrupted_status));
+
+  if (old_interrupted_status & SVM_THREAD_THROW_INTERRUPTED)
+    {
+      _svmm_resuming_java (env);
+      _svmf_error_InterruptedException (env);
+      _svmm_stopping_java (env);
+    }
+}
+
 /*
 ----------------------------------------------------------------------
 Java_java_lang_VMThread_nativeIsAlive
@@ -52,18 +130,18 @@ Java_java_lang_VMThread_nativeIsAlive
  * Signature: ([B)Z
  */
 
-static JNIEXPORT jboolean JNICALL
+JNIEXPORT static jboolean JNICALL
 Java_java_lang_VMThread_nativeIsAlive (JNIEnv *_env, jobject this SVM_UNUSED,
 				       jbyteArray vmData)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
   jboolean result;
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   result = ((_svmt_JNIEnv *) _svmf_unwrap_pointer (*vmData))->is_alive;
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 
   return result;
 }
@@ -80,152 +158,31 @@ Java_java_lang_VMThread_nativeStart
  * Signature: ()[B
  */
 
-static JNIEXPORT jbyteArray JNICALL
+JNIEXPORT static jbyteArray JNICALL
 Java_java_lang_VMThread_nativeStart (JNIEnv *_env, jobject this SVM_UNUSED,
 				     jobject threadInstance)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
   jbyteArray result = NULL;
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
-    _svmt_JavaVM *vm = env->vm;
-    _svmt_JNIEnv *new_env = NULL;;
+    _svmt_JNIEnv *new_env = _svmf_thread_native_start (env, threadInstance);
 
-    {
-      jint status = JNI_OK;
-
-      _svmm_mutex_lock (vm->global_mutex);
-
-      _svmf_halt_if_requested (env);
-
-      if (vm->threads.free_list != NULL)
-	{
-	  new_env = vm->threads.free_list;
-	  assert (new_env->previous == NULL);
-
-	  vm->threads.free_list = new_env->next;
-	  if (vm->threads.free_list != NULL)
-	    {
-	      vm->threads.free_list->previous = NULL;
-	    }
-
-	  new_env->next = vm->threads.user;
-	  if (new_env->next != NULL)
-	    {
-	      assert (new_env->next->previous == NULL);
-	      new_env->next->previous = new_env;
-	    }
-
-	  new_env->thread_status = SVM_THREAD_STATUS_RUNNING_JAVA;
-	}
-      else if (vm->threads.next_thread_id <= SVM_MAX_THREAD_ID)
-	{
-	  if (_svmm_gzalloc_env_no_exception (new_env) != JNI_OK)
-	    {
-	      goto unlock;
-	    }
-
-	  new_env->interface = &_svmv_native_interface;
-	  new_env->vm = vm;
-
-	  new_env->next = vm->threads.user;
-	  vm->threads.user = new_env;
-	  if (new_env->next != NULL)
-	    {
-	      assert (new_env->next->previous == NULL);
-	      new_env->next->previous = new_env;
-	    }
-
-	  new_env->thread.interrupted_status = SVM_THREAD_NOT_INTERRUPTED;
-	  new_env->thread.sleeping_on_fat_lock = NULL;
-
-	  new_env->thread.id = vm->threads.next_thread_id++;
-	  _svmf_initialize_thinlock_id (new_env);
-	  vm->threads.array[new_env->thread.id] = new_env;
-
-	  _svmm_mutex_init (new_env->contention.owner.mutex);
-	  _svmm_cond_init (new_env->contention.requester.cond);
-
-	  new_env->thread_status = SVM_THREAD_STATUS_RUNNING_JAVA;
-	}
-      else
-	{
-	  status = JNI_ERR;
-	}
-
-    unlock:
-      _svmm_mutex_unlock ();
-
-      if (status != JNI_OK)
-	{
-	  _svmf_error_OutOfMemoryError (env);
-	  goto end;
-	}
-    }
-
-    if (_svmm_gzalloc_native_ref_no_exception (new_env->native_locals.list) !=
-	JNI_OK)
+    if (new_env != NULL)
       {
-	/* we should be cleaning up!  todo ... */
-	_svmf_error_OutOfMemoryError (env);
-	goto end;
-      }
-    new_env->throwable =
-      _svmf_cast_jobject_nref (new_env->native_locals.list);
-
-    if (_svmm_gzalloc_native_ref_no_exception
-	(new_env->native_locals.list->next) != JNI_OK)
-      {
-	/* we should be cleaning up!  todo ... */
-	_svmf_error_OutOfMemoryError (env);
-	goto end;
-      }
-    new_env->thread.thread_instance =
-      _svmf_cast_jobject_nref (new_env->native_locals.list->next);
-    *(new_env->thread.thread_instance) = *(threadInstance);
-
-    if (_svmm_gzalloc_native_ref_no_exception
-	(new_env->native_locals.list->next->next) != JNI_OK)
-      {
-	/* we should be cleaning up!  todo ... */
-	_svmf_error_OutOfMemoryError (env);
-	goto end;
-      }
-    new_env->contention.requester.jobject =
-      _svmf_cast_jobject_nref (new_env->native_locals.list->next->next);
-
-    if (_svmf_stack_init (new_env) != JNI_OK)
-      {
-	/* we should be cleaning up!  todo ... */
-	*(env->throwable) = *(new_env->throwable);
-	goto end;
-      }
-
-    /* start it! */
-    new_env->is_alive = JNI_TRUE;
-    if (pthread_create
-	(&new_env->thread.pthread, NULL, &_svmf_thread_start, new_env) != 0)
-      {
-	/* something went wrong */
-	/* we should be cleaning up!  todo ... */
-	new_env->is_alive = JNI_FALSE;
-	_svmf_error_InternalError (env);
-	goto end;
-      }
-
-    /* it's started! */
-    result = _svmf_get_jni_frame_native_local_array (env);
-    if (_svmf_wrap_pointer (env, new_env, result) != JNI_OK)
-      {
-	result = NULL;
-	goto end;
+	result = _svmf_get_jni_frame_native_local_array (env);
+	if (_svmf_wrap_pointer (env, new_env, result) != JNI_OK)
+	  {
+	    result = NULL;
+	    goto end;
+	  }
       }
   }
 
 end:
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 
   return result;
 }
@@ -242,13 +199,13 @@ Java_java_lang_VMThread_currentThread
  * Signature: ()Ljava/lang/Thread;
  */
 
-static JNIEXPORT jobject JNICALL
+JNIEXPORT static jobject JNICALL
 Java_java_lang_VMThread_currentThread (JNIEnv *_env, jclass _class SVM_UNUSED)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
   jobject result = NULL;
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     if (env->thread.thread_instance == NULL)
@@ -284,7 +241,7 @@ Java_java_lang_VMThread_currentThread (JNIEnv *_env, jclass _class SVM_UNUSED)
   }
 
 end:
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 
   return result;
 }
@@ -301,7 +258,7 @@ Java_java_lang_VMThread_nativeInterrupt
  * Method:    interrupt
  * Signature: ()V
  */
-static JNIEXPORT void JNICALL
+JNIEXPORT static void JNICALL
 Java_java_lang_VMThread_nativeInterrupt (JNIEnv *_env,
 					 jobject this SVM_UNUSED,
 					 jbyteArray vmData)
@@ -312,7 +269,7 @@ Java_java_lang_VMThread_nativeInterrupt (JNIEnv *_env,
   _svmt_fat_lock *sleeping_on_fat_lock;
   _svmt_word old_interrupted_status, new_interrupted_status;
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
 again:
 
@@ -392,7 +349,7 @@ again:
       _svmm_kill (target_env->thread.pthread, SVM_INTERRUPT_SIGNAL);
     }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 
   return;
 }
@@ -408,7 +365,7 @@ Java_java_lang_VMThread_nativeIsInterrupted
  * Method:    isInterrupted
  * Signature: ()Z
  */
-static JNIEXPORT jboolean JNICALL
+JNIEXPORT static jboolean JNICALL
 Java_java_lang_VMThread_nativeIsInterrupted (JNIEnv *_env,
 					     jobject this SVM_UNUSED,
 					     jbyteArray vmData)
@@ -416,13 +373,13 @@ Java_java_lang_VMThread_nativeIsInterrupted (JNIEnv *_env,
   jboolean result;
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   result =
     (((_svmt_JNIEnv *) _svmf_unwrap_pointer (*vmData))->thread.
      interrupted_status) & SVM_THREAD_INTERRUPTED;
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 
   return result;
 }
@@ -438,14 +395,14 @@ Java_java_lang_VMThread_interrupted
  * Method:    interrupted
  * Signature: ()Z
  */
-static JNIEXPORT jboolean JNICALL
+JNIEXPORT static jboolean JNICALL
 Java_java_lang_VMThread_interrupted (JNIEnv *_env, jclass this SVM_UNUSED)
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
   _svmt_word old_interrupted_status, new_interrupted_status;
   jboolean result;
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   /* this is static method that refers to the _current_ thread only,
      so no need to search which thread is referred to by "this"
@@ -464,7 +421,7 @@ Java_java_lang_VMThread_interrupted (JNIEnv *_env, jclass this SVM_UNUSED)
 
   result = old_interrupted_status & SVM_THREAD_INTERRUPTED;
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 
   return result;
 }
@@ -480,7 +437,7 @@ Java_java_lang_VMThread_holdsLock
  * Method:    holdsLock 
  * Signature: (Ljava/lang/Object;)Z 
  */
-static JNIEXPORT jboolean JNICALL
+JNIEXPORT static jboolean JNICALL
 Java_java_lang_VMThread_holdsLock (JNIEnv *_env, jclass class SVM_UNUSED,
 				   jobject o)
 {
@@ -488,7 +445,7 @@ Java_java_lang_VMThread_holdsLock (JNIEnv *_env, jclass class SVM_UNUSED,
   _svmt_JavaVM *vm = env->vm;
   jboolean result = JNI_TRUE;
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     /* Note: the case where o == NULL should be handled before calling this function */
@@ -519,7 +476,79 @@ Java_java_lang_VMThread_holdsLock (JNIEnv *_env, jclass class SVM_UNUSED,
   }
 
 end:
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 
   return result;
 }
+
+/*
+----------------------------------------------------------------------
+Java_java_lang_VMThread_suspend
+----------------------------------------------------------------------
+*/
+
+/*
+ * Class:     java_lang_VMThread
+ * Method:    suspend
+ * Signature: ()V
+ */
+
+JNIEXPORT static void JNICALL
+Java_java_lang_VMThread_suspend (JNIEnv *_env, jclass class SVM_UNUSED,
+				 jbyteArray vmData)
+{
+  _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
+  _svmt_JavaVM *vm = env->vm;
+
+  _svmm_resuming_java (env);
+
+  {
+    _svmt_JNIEnv *target;
+
+    target = _svmf_unwrap_pointer (*vmData);
+
+    _svmm_mutex_lock (vm->global_mutex);
+
+    _svmf_suspend_thread (env, target);
+
+    _svmm_mutex_unlock ();
+  }
+
+  _svmm_stopping_java (env);
+}
+
+/*
+----------------------------------------------------------------------
+Java_java_lang_VMThread_resume
+----------------------------------------------------------------------
+*/
+
+/*
+ * Class:     java_lang_VMThread
+ * Method:    resume
+ * Signature: ()V
+ */
+
+JNIEXPORT static void JNICALL
+Java_java_lang_VMThread_resume (JNIEnv *_env, jclass class SVM_UNUSED,
+				jbyteArray vmData)
+{
+  _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
+  _svmt_JavaVM *vm = env->vm;
+
+  _svmm_resuming_java (env);
+
+  {
+    _svmt_JNIEnv *target;
+
+    target = _svmf_unwrap_pointer (*vmData);
+
+    _svmm_mutex_lock (vm->global_mutex);
+
+    _svmf_resume_thread (env, target);
+
+    _svmm_mutex_unlock ();
+  }
+
+  _svmm_stopping_java (env);
+}
