@@ -25,7 +25,7 @@ Java_java_lang_VirtualMachine_invokeMain (JNIEnv *_env,
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     _svmt_type_info *type = _svmf_unwrap_class_instance (env, mainClass);
@@ -67,7 +67,7 @@ Java_java_lang_VirtualMachine_invokeMain (JNIEnv *_env,
   }
 
 end:
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 }
 
 /*
@@ -88,13 +88,13 @@ Java_java_lang_VirtualMachine_print__Z (JNIEnv *_env, jclass class SVM_UNUSED,
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     _svmf_printf (env, stderr, "%s", value ? "true" : "false");
   }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 }
 
 /*
@@ -115,13 +115,13 @@ Java_java_lang_VirtualMachine_print__B (JNIEnv *_env, jclass class SVM_UNUSED,
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     _svmf_printf (env, stderr, "%d", (jint) value);
   }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 }
 
 /*
@@ -142,13 +142,13 @@ Java_java_lang_VirtualMachine_print__S (JNIEnv *_env, jclass class SVM_UNUSED,
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     _svmf_printf (env, stderr, "%d", (jint) value);
   }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 }
 
 /*
@@ -169,7 +169,7 @@ Java_java_lang_VirtualMachine_print__C (JNIEnv *_env, jclass class SVM_UNUSED,
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     if (value > 0x7f || value < ' ')
@@ -183,7 +183,7 @@ Java_java_lang_VirtualMachine_print__C (JNIEnv *_env, jclass class SVM_UNUSED,
     _svmf_printf (env, stderr, "%c", (jint) value);
   }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 }
 
 /*
@@ -204,13 +204,13 @@ Java_java_lang_VirtualMachine_print__I (JNIEnv *_env, jclass class SVM_UNUSED,
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     _svmf_printf (env, stderr, "%d", value);
   }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 }
 
 /*
@@ -231,13 +231,13 @@ Java_java_lang_VirtualMachine_print__J (JNIEnv *_env, jclass class SVM_UNUSED,
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     _svmf_printf (env, stderr, "%lld", value);
   }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 }
 
 /*
@@ -258,13 +258,13 @@ Java_java_lang_VirtualMachine_print__F (JNIEnv *_env, jclass class SVM_UNUSED,
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     _svmf_printf (env, stderr, "%f", (jdouble) value);
   }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 }
 
 /*
@@ -285,13 +285,13 @@ Java_java_lang_VirtualMachine_print__D (JNIEnv *_env, jclass class SVM_UNUSED,
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     _svmf_printf (env, stderr, "%f", value);
   }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 
 }
 
@@ -315,7 +315,7 @@ Java_java_lang_VirtualMachine_print__Ljava_lang_String_2 (JNIEnv *_env,
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   if (value != NULL)
     {
@@ -336,7 +336,7 @@ Java_java_lang_VirtualMachine_print__Ljava_lang_String_2 (JNIEnv *_env,
     }
 
 end:
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 }
 
 /*
@@ -358,13 +358,13 @@ Java_java_lang_VirtualMachine_println__Z (JNIEnv *_env,
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     _svmf_printf (env, stderr, "%s\n", value ? "true" : "false");
   }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 }
 
 /*
@@ -386,13 +386,13 @@ Java_java_lang_VirtualMachine_println__B (JNIEnv *_env,
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     _svmf_printf (env, stderr, "%d\n", (jint) value);
   }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 }
 
 /*
@@ -414,13 +414,13 @@ Java_java_lang_VirtualMachine_println__S (JNIEnv *_env,
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     _svmf_printf (env, stderr, "%d\n", (jint) value);
   }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 }
 
 /*
@@ -442,7 +442,7 @@ Java_java_lang_VirtualMachine_println__C (JNIEnv *_env,
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     if (value > 0x7f || value < ' ')
@@ -456,7 +456,7 @@ Java_java_lang_VirtualMachine_println__C (JNIEnv *_env,
     _svmf_printf (env, stderr, "%c\n", (jint) value);
   }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 }
 
 /*
@@ -477,13 +477,13 @@ Java_java_lang_VirtualMachine_println__I (JNIEnv *_env,
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     _svmf_printf (env, stderr, "%d\n", value);
   }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 }
 
 /*
@@ -505,13 +505,13 @@ Java_java_lang_VirtualMachine_println__J (JNIEnv *_env,
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     _svmf_printf (env, stderr, "%lld\n", value);
   }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 }
 
 /*
@@ -533,13 +533,13 @@ Java_java_lang_VirtualMachine_println__F (JNIEnv *_env,
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     _svmf_printf (env, stderr, "%f\n", (jdouble) value);
   }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 }
 
 /*
@@ -561,13 +561,13 @@ Java_java_lang_VirtualMachine_println__D (JNIEnv *_env,
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     _svmf_printf (env, stderr, "%f\n", value);
   }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 }
 
 /*
@@ -590,7 +590,7 @@ Java_java_lang_VirtualMachine_println__Ljava_lang_String_2 (JNIEnv *_env,
 {
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   if (value != NULL)
     {
@@ -611,5 +611,5 @@ Java_java_lang_VirtualMachine_println__Ljava_lang_String_2 (JNIEnv *_env,
     }
 
 end:
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 }
