@@ -24,7 +24,7 @@ Java_java_lang_reflect_Array_createObjectArray (JNIEnv *_env,
   jobjectArray instance = NULL;
   _svmt_array_info *array_info;
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     _svmt_type_info *type = _svmf_unwrap_class_instance (env, _class);
@@ -62,7 +62,7 @@ Java_java_lang_reflect_Array_createObjectArray (JNIEnv *_env,
   }
 
 end:
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 
   return _svmf_cast_jobject (instance);
 
