@@ -87,7 +87,7 @@ Java_java_lang_reflect_Constructor_constructNative (JNIEnv *_env,
   jobject current_exception = NULL;
   jobject this = NULL;
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     _svmt_method_info *constructor = _svmf_unwrap_pointer (*vmData);
@@ -345,7 +345,7 @@ end:
       _svmm_free_native_local (env, current_exception);
     }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 
   return this;
 }
@@ -371,7 +371,7 @@ Java_java_lang_reflect_Constructor_nativeGetDescriptor (JNIEnv *_env,
   _svmt_JNIEnv *env = _svmf_cast_svmt_JNIEnv (_env);
   jstring descriptor;
 
-  _svmf_resuming_java (env);
+  _svmm_resuming_java (env);
 
   {
     _svmt_method_info *constructor = _svmf_unwrap_pointer (*vmData);
@@ -380,7 +380,7 @@ Java_java_lang_reflect_Constructor_nativeGetDescriptor (JNIEnv *_env,
     _svmf_get_string (env, DREF (constructor->descriptor, value), descriptor);
   }
 
-  _svmf_stopping_java (env);
+  _svmm_stopping_java (env);
 
   return descriptor;
 }
