@@ -941,7 +941,7 @@ Java_java_lang_reflect_Field_nativeSetReference (JNIEnv *_env,
 	size_t offset;
 	assert (instance != NULL);
 	offset = field->data.instance_field.offset;
-	_svmf_put_REFERENCE_field (*instance, offset, *value);
+	_svmf_put_REFERENCE_field (env, *instance, offset, *value);
       }
   }
 
