@@ -263,6 +263,10 @@ _svmf_get_jni_frame_native_local
 ----------------------------------------------------------------------
 */
 
+/* It is CRITICAL that this function does not cause garbage
+ * collection, or, if it does, it MUST abort execution.
+ */
+
 /* NOTE: IT IS IMPORTANT to assign a value OTHER THAN NULL into the
  * returned jobject BEFORE calling this method again. IF YOU DON'T
  * follow rigorously this rule, you will probably experience problems,
@@ -272,7 +276,7 @@ _svmf_get_jni_frame_native_local
  * return values (as would be necessary with _svmf_new_native_local),
  * by using stack frame reserved JNI jobjects.  Its primary function
  * is to help implement the JNI mechanism.  So, if you're not sure,
- * simply use _svmf_new_native_local; you'll then be safe.
+ * simply use _svmm_new_native_local; you'll then be safe.
  */
 
 svm_static jobject
@@ -323,14 +327,26 @@ _svmf_get_jni_frame_native_local (_svmt_JNIEnv *env)
 			method->data.native_method->refargs_count);
 	}
 
+      /* The following can change env->stack.start/end/current_frame
+         and potentially cause GC on failure to create an Exception
+         instance. */
       if (_svmf_ensure_native_local_capacity
 	  (env, SVM_FRAME_NATIVE_REFS_AUTOADD_STEP) != JNI_OK)
 	{
 	  goto end;
 	}
 
+      frame = env->stack.current_frame;
+      lrefs = (_svmt_stack_native_reference *)
+	(void *) (((char *) frame) + frame->end_offset);
+      lrefs_size = lrefs[-2].size_t;
+      lrefs =
+	(_svmt_stack_native_reference *) (void *) (((char *) lrefs) -
+						   lrefs_size);
+
+
       /* get the reference again, it must be available this time */
-      return _svmf_get_jni_frame_native_local (env);
+      return lrefs[i].jobject;
     }
 
 #endif
