@@ -680,6 +680,14 @@ _svmf_bind_native_method (_svmt_JNIEnv *env, _svmt_method_info *method)
 	  goto error;
 	}
 
+      /* Highly useful, when all you see is java/lang/UnsatisfiedLinkError. */
+      if (vm->verbose_jni)
+	{
+	  _svmf_printf (env, stderr,
+			"[verbose jni: Unable to link native method %s]\n",
+			method->data.native_method->short_name);
+	}
+
       _svmf_error_UnsatisfiedLinkError_msg (env, function_name);
 
       _svmm_free_native_local (env, function_name);
@@ -791,6 +799,45 @@ _svmf_invoke_native_static (_svmt_JNIEnv *env)
       }
   }
 
+
+#ifndef NDEBUG
+
+  {
+    /* check argument types */
+
+    jint arg_count = method->java_args_count;
+    const char *pdesc = DREF (method->descriptor, value) + 1;
+    jint arg = 0;
+
+    while (arg < arg_count)
+      {
+	jint old_arg = arg;
+
+	if (params[old_arg].reference != NULL)
+	  {
+	    _svmt_type_info *t =
+	      _svmm_get_type_from_within_descriptor (env, method, pdesc, arg);
+
+	    if (t != NULL)
+	      {
+		if (!_svmf_is_assignable_from
+		    (env, params[old_arg].reference->vtable->type, t))
+		  {
+		    _svmf_dump_stack_trace (env);
+		    _svmm_fatal_error ("argument type bug detected");
+		  }
+	      }
+	  }
+	else
+	  {
+	    _svmm_skip_type_within_descriptor (pdesc, arg);
+	  }
+      }
+  }
+
+#endif
+
+
   /* time to initialize ptrs array */
   assert (ref == 0);
   assert (param == 0);
@@ -1107,6 +1154,35 @@ _svmf_invoke_native_static (_svmt_JNIEnv *env)
 		  obj = *((jobject *) &result[0]);
 		}
 
+
+#ifndef NDEBUG
+
+	      if (obj != NULL)
+		{
+		  /* check return value type */
+
+		  const char *pdesc = DREF (method->descriptor, value) + 1;
+		  jint arg = 0;
+		  _svmt_type_info *t;
+
+		  while (*pdesc++ != ')');
+
+		  t =
+		    _svmm_get_type_from_within_descriptor (env, method, pdesc,
+							   arg);
+
+		  assert (t != NULL);
+
+		  if (*obj == NULL || !_svmf_is_assignable_from
+		      (env, (*obj)->vtable->type, t))
+		    {
+		      _svmf_dump_stack_trace (env);
+		      _svmm_fatal_error ("argument type bug detected");
+		    }
+		}
+
+#endif
+
 	      if (obj != NULL)
 		{
 		  params[0].reference = *obj;
@@ -1270,6 +1346,56 @@ _svmf_invoke_native_nonstatic (_svmt_JNIEnv *env)
       }
   }
 
+#ifndef NDEBUG
+
+  {
+    /* check argument types */
+
+    jint arg_count = method->java_args_count;
+    const char *pdesc = DREF (method->descriptor, value) + 1;
+    jint arg = 0;
+
+    while (arg < arg_count)
+      {
+	jint old_arg = arg;
+
+	if (params[old_arg].reference != NULL)
+	  {
+	    _svmt_type_info *t;
+
+	    if (old_arg == 0)
+	      {
+		t = _svmf_cast_type_class (method->class_info);
+		arg++;
+	      }
+	    else
+	      {
+		t =
+		  _svmm_get_type_from_within_descriptor (env, method, pdesc,
+							 arg);
+	      }
+
+	    if (t != NULL)
+	      {
+		if (!_svmf_is_assignable_from
+		    (env, params[old_arg].reference->vtable->type, t))
+		  {
+		    _svmf_dump_stack_trace (env);
+		    _svmm_fatal_error ("argument type bug detected");
+		  }
+	      }
+	  }
+	else
+	  {
+	    assert (old_arg != 0);
+
+	    _svmm_skip_type_within_descriptor (pdesc, arg);
+	  }
+      }
+  }
+
+#endif
+
   /* time to initialize ptrs array */
   assert (ref == 0);
   assert (param == 0);
@@ -1588,6 +1714,33 @@ _svmf_invoke_native_nonstatic (_svmt_JNIEnv *env)
 		  obj = *((jobject *) &result[0]);
 		}
 
+#ifndef NDEBUG
+
+	      if (obj != NULL)
+		{
+		  /* check return value type */
+
+		  const char *pdesc = DREF (method->descriptor, value) + 1;
+		  jint arg = 0;
+		  _svmt_type_info *t;
+
+		  while (*pdesc++ != ')');
+
+		  t =
+		    _svmm_get_type_from_within_descriptor (env, method, pdesc,
+							   arg);
+
+		  assert (t != NULL);
+
+		  if (*obj == NULL || !_svmf_is_assignable_from
+		      (env, (*obj)->vtable->type, t))
+		    {
+		      _svmf_dump_stack_trace (env);
+		      _svmm_fatal_error ("argument type bug detected");
+		    }
+		}
+#endif
+
 	      if (obj != NULL)
 		{
 		  params[0].reference = *obj;
