@@ -12,7 +12,7 @@ _svmf_resolve_CONSTANT_Class
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_resolve_CONSTANT_Class (_svmt_JNIEnv *env, _svmt_class_info *class,
 			      _svmt_CONSTANT_Class_info *classref)
 {
@@ -23,10 +23,8 @@ _svmf_resolve_CONSTANT_Class (_svmt_JNIEnv *env, _svmt_class_info *class,
 
   if (vm->initialization == NULL)
     {
-      if (_svmf_enter_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+      if (_svmf_enter_class_loader_monitor (env, class->class_loader_info) !=
+	  JNI_OK)
 	{
 	  goto error;
 	}
@@ -68,10 +66,8 @@ ok:
     {
       monitor_acquired = JNI_FALSE;
 
-      if (_svmf_exit_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+      if (_svmf_exit_class_loader_monitor (env, class->class_loader_info) !=
+	  JNI_OK)
 	{
 	  goto error;
 	}
@@ -85,10 +81,8 @@ error:
     {
       monitor_acquired = JNI_FALSE;
 
-      if (_svmf_exit_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+      if (_svmf_exit_class_loader_monitor (env, class->class_loader_info) !=
+	  JNI_OK)
 	{
 	  goto error;
 	}
@@ -103,7 +97,7 @@ _svmf_resolve_field
 ----------------------------------------------------------------------
 */
 
-static _svmt_field_info *
+svm_static _svmt_field_info *
 _svmf_resolve_field (_svmt_class_info *class, const char *name,
 		     const char *descriptor)
 {
@@ -161,22 +155,17 @@ _svmf_resolve_CONSTANT_Fieldref
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_resolve_CONSTANT_Fieldref (_svmt_JNIEnv *env, _svmt_class_info *class,
 				 _svmt_CONSTANT_Fieldref_info *fieldref)
 {
-  _svmt_field_info *field;
-
-
   _svmt_JavaVM *vm = env->vm;
   jboolean monitor_acquired = JNI_FALSE;
 
   if (vm->initialization == NULL)
     {
-      if (_svmf_enter_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+      if (_svmf_enter_object_monitor (env, *(class->class_instance)) !=
+	  JNI_OK)
 	{
 	  goto error;
 	}
@@ -244,10 +233,7 @@ ok:
     {
       monitor_acquired = JNI_FALSE;
 
-      if (_svmf_exit_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+      if (_svmf_exit_object_monitor (env, *(class->class_instance)) != JNI_OK)
 	{
 	  goto error;
 	}
@@ -261,10 +247,7 @@ error:
     {
       monitor_acquired = JNI_FALSE;
 
-      if (_svmf_exit_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+      if (_svmf_exit_object_monitor (env, *(class->class_instance)) != JNI_OK)
 	{
 	  goto error;
 	}
@@ -279,7 +262,7 @@ _svmf_resolve_method
 ----------------------------------------------------------------------
 */
 
-static _svmt_method_info *
+svm_static _svmt_method_info *
 _svmf_resolve_method (_svmt_class_info *class, const char *name,
 		      const char *descriptor)
 {
@@ -353,7 +336,7 @@ _svmf_resolve_incomplete_method
 ----------------------------------------------------------------------
 */
 
-static _svmt_method_info *
+svm_static _svmt_method_info *
 _svmf_resolve_incomplete_method (_svmt_class_info *class, const char *name,
 				 const char *incomplete_descriptor)
 {
@@ -419,22 +402,17 @@ _svmf_resolve_CONSTANT_Methodref
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_resolve_CONSTANT_Methodref (_svmt_JNIEnv *env, _svmt_class_info *class,
 				  _svmt_CONSTANT_Methodref_info *methodref)
 {
-  _svmt_method_info *method;
-
-
   _svmt_JavaVM *vm = env->vm;
   jboolean monitor_acquired = JNI_FALSE;
 
   if (vm->initialization == NULL)
     {
-      if (_svmf_enter_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+      if (_svmf_enter_object_monitor (env, *(class->class_instance)) !=
+	  JNI_OK)
 	{
 	  goto error;
 	}
@@ -503,10 +481,7 @@ ok:
     {
       monitor_acquired = JNI_FALSE;
 
-      if (_svmf_exit_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+      if (_svmf_exit_object_monitor (env, *(class->class_instance)) != JNI_OK)
 	{
 	  goto error;
 	}
@@ -520,10 +495,7 @@ error:
     {
       monitor_acquired = JNI_FALSE;
 
-      if (_svmf_exit_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+      if (_svmf_exit_object_monitor (env, *(class->class_instance)) != JNI_OK)
 	{
 	  goto error;
 	}
@@ -538,7 +510,7 @@ _svmf_resolve_imethod
 ----------------------------------------------------------------------
 */
 
-static _svmt_method_info *
+svm_static _svmt_method_info *
 _svmf_resolve_imethod (_svmt_class_info *class, const char *name,
 		       const char *descriptor)
 {
@@ -586,24 +558,19 @@ _svmf_resolve_CONSTANT_InterfaceMethodref
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_resolve_CONSTANT_InterfaceMethodref (_svmt_JNIEnv *env,
 					   _svmt_class_info *class,
 					   _svmt_CONSTANT_InterfaceMethodref_info
 					   *imethodref)
 {
-  _svmt_method_info *method;
-
-
   _svmt_JavaVM *vm = env->vm;
   jboolean monitor_acquired = JNI_FALSE;
 
   if (vm->initialization == NULL)
     {
-      if (_svmf_enter_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+      if (_svmf_enter_object_monitor (env, *(class->class_instance)) !=
+	  JNI_OK)
 	{
 	  goto error;
 	}
@@ -672,10 +639,7 @@ ok:
     {
       monitor_acquired = JNI_FALSE;
 
-      if (_svmf_exit_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+      if (_svmf_exit_object_monitor (env, *(class->class_instance)) != JNI_OK)
 	{
 	  goto error;
 	}
@@ -689,10 +653,7 @@ error:
     {
       monitor_acquired = JNI_FALSE;
 
-      if (_svmf_exit_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+      if (_svmf_exit_object_monitor (env, *(class->class_instance)) != JNI_OK)
 	{
 	  goto error;
 	}
@@ -707,7 +668,7 @@ _svmf_resolve_CONSTANT_String
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_resolve_CONSTANT_String (_svmt_JNIEnv *env,
 			       _svmt_CONSTANT_String_info *stringref)
 {
@@ -719,9 +680,7 @@ _svmf_resolve_CONSTANT_String (_svmt_JNIEnv *env,
   if (vm->initialization == NULL)
     {
       if (_svmf_enter_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+	  (env, *(stringref->class_info->class_instance)) != JNI_OK)
 	{
 	  goto error;
 	}
@@ -756,9 +715,7 @@ ok:
       monitor_acquired = JNI_FALSE;
 
       if (_svmf_exit_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+	  (env, *(stringref->class_info->class_instance)) != JNI_OK)
 	{
 	  goto error;
 	}
@@ -773,9 +730,7 @@ error:
       monitor_acquired = JNI_FALSE;
 
       if (_svmf_exit_object_monitor
-	  (env,
-	   *(vm->class_loading.boot_loader.classes.virtualmachine->
-	     class_instance)) != JNI_OK)
+	  (env, *(stringref->class_info->class_instance)) != JNI_OK)
 	{
 	  goto error;
 	}
@@ -790,7 +745,7 @@ _svmf_resolve_special_method
 ----------------------------------------------------------------------
 */
 
-static _svmt_method_info *
+svm_static _svmt_method_info *
 _svmf_resolve_special_method (_svmt_class_info *class, const char *name,
 			      const char *descriptor)
 {
