@@ -18,7 +18,7 @@ extern void __clear_cache (char *beg, char *end);
     || defined (__i386__) || defined (__powerpc__) || defined (__s390__) \
     || defined (__hppa__) || defined (__arm__) || defined (__m68k__) \
     || defined (__mc68000__) || defined (__mips__) || defined (__mipsel__) \
-    || defined (__x86_64__) \
+    || defined (__x86_64__) || defined (_POWER)  \
      ) && defined (__GNUC__))
 
 /*
@@ -63,7 +63,7 @@ _svmf_iflush (_svmt_word *pword SVM_UNUSED)
   __asm__ __volatile__ ("fc %0;;; sync.i;;; mf;;; srlz.i;;;"::
 			"r" (pword):"memory");
 
-#elif defined (__powerpc__)
+#elif defined (__powerpc__) || defined (_POWER)
 
   __asm__ __volatile__ ("dcbst 0,%0\n\t"
 			"sync\n\t"
@@ -296,6 +296,32 @@ _svmh_compare_and_swap (volatile _svmt_word *pword, _svmt_word old_value,
   /* flip the value of result */
   result = !result;
 
+#elif defined (_POWER)
+
+  int result;
+
+  /*
+   * Adapted from the glibc-linuxthreads, file pt-machine.h
+   */
+
+/* *INDENT-OFF* */
+  __asm__ __volatile__ ("SVM0:    lwarx %0,0,%1\n\t"
+			"      xor. %0,%3,%0\n\t"
+			"      bne SVM1\n\t"
+			"      stwcx. %2,0,%1\n\t"
+			"      bne- SVM0\n\t"
+			"SVM1:    ":"=&r" (result):"r" (pword), "r" (new_value),
+			"r" (old_value):"cr0", "memory");
+  /* This version of __compare_and_swap is to be used when acquiring
+     a lock, so we don't need to worry about whether other memory
+     operations have completed, but we do need to be sure that any loads
+     after this point really occur after we have acquired the lock.  */
+  __asm__ __volatile__ ("isync" : : : "memory");
+/* *INDENT-ON* */
+
+  /* flip the value of result */
+  result = !result;
+
 #elif defined (__s390__)
 
   /* borrowed from glibc so is LGPL - file atomicity.h for s390-32 (GBP) */
