@@ -55,7 +55,7 @@ int main (int argc, char *argv[])
     || defined (__i386__) || defined (__powerpc__) || defined (__s390__) \
     || defined (__hppa__) || defined (__arm__) || defined (__m68k__) \
     || defined (__mc68000__) || defined (__mips__) || defined (__mipsel__) \
-    || defined (__x86_64__) \
+    || defined (__x86_64__) || defined (_POWER)       \
      ) && defined (__GNUC__))
 
 /* "inline" is now an official keyword since the latest C standard (1999).
