@@ -500,6 +500,9 @@ struct _svmt_zip_file_entry_struct
 {
   char *name;
   _svmt_u32 local_header_offset;
+  _svmt_u32 size_compressed;
+  _svmt_u32 size_uncompressed;
+  _svmt_u32 compression_method;
 };
 
 /*
