@@ -6,6 +6,8 @@
  * modification of this source file.                               *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
+#include "gc_gencopy.h"
+
 /*
 ----------------------------------------------------------------------
 _svmf_bytes_to_u16
@@ -468,11 +470,16 @@ _svmf_put_REFERENCE_field
 */
 
 inline static void
-_svmf_put_REFERENCE_field (_svmt_object_instance *instance, size_t offset,
+_svmf_put_REFERENCE_field (_svmt_JNIEnv *env SVM_UNUSED,
+			   _svmt_object_instance *instance, size_t offset,
 			   _svmt_object_instance *value)
 {
   *((_svmt_object_instance **) (void *) (((char *) instance) + offset)) =
     value;
+#if defined (_SABLEVM_GC_WRITE_BARRIER)
+  _svmf_write_barrier
+    (env, (_svmt_object_instance **) (void *) ((char *) instance + offset));
+#endif /* _SABLEVM_GC_WRITE_BARRIER */
 }
 
 /*
