@@ -1522,7 +1522,7 @@ _svmf_wrap_exception_in_invocationte (_svmt_JNIEnv *env,
 
 /*
 ----------------------------------------------------------------------
-_svmf_verbose_methods_enter (_svmt_JNIEnv *env)
+_svmf_verbose_methods_enter
 ----------------------------------------------------------------------
 */
 
@@ -1548,7 +1548,7 @@ _svmf_verbose_methods_enter (_svmt_JNIEnv *env)
 
 /*
 ----------------------------------------------------------------------
-_svmf_verbose_methods_exit (_svmt_JNIEnv *env)
+_svmf_verbose_methods_exit
 ----------------------------------------------------------------------
 */
 
@@ -1600,6 +1600,236 @@ _svmf_verbose_methods_exit (_svmt_JNIEnv *env)
 
 #endif /* !NDEBUG */
 
+/*
+----------------------------------------------------------------------
+_svmm_get_type_from_within_descriptor
+----------------------------------------------------------------------
+*/
+
+#ifndef NDEBUG
+
+#define _svmm_get_type_from_within_descriptor(env, method_info, current, index) \
+_svmh_get_type_from_within_descriptor (env, method_info, &current, &index)
+
+svm_static _svmt_type_info *
+_svmh_get_type_from_within_descriptor (_svmt_JNIEnv *env,
+				       _svmt_method_info *method_info,
+				       const char **pcurrent, jint *pindex)
+{
+  const char *current = *pcurrent;
+  _svmt_type_info *result = NULL;
+
+  switch (*current)
+    {
+    case 'B':
+    case 'C':
+    case 'F':
+    case 'I':
+    case 'S':
+    case 'Z':
+    case 'V':
+      {
+	current++;
+	(*pindex)++;
+      }
+      break;
+
+    case 'D':
+    case 'J':
+      {
+	current++;
+	(*pindex) += 2;
+      }
+      break;
+
+    case 'L':
+      {
+	jint length = 0;
+	char *p = (char *) current;
+	_svmt_u8 *name;
+
+	while (*++p != ';')
+	  {
+	    length++;
+	  }
+
+	if (_svmm_gzmalloc_ubytes (env, length + 1, name) != JNI_OK)
+	  {
+	    _svmm_fatal_error ("unsupported failure while debugging");
+	  }
+
+	p = (char *) name;
+	while (*++current != ';')
+	  {
+	    *(p++) = *current;
+	  }
+
+	current++;
+	(*pindex)++;
+
+	if (_svmm_create_type
+	    (env, method_info->class_info->class_loader_info,
+	     (const char *) name, result) != JNI_OK)
+	  {
+	    _svmm_fatal_error ("unsupported failure while debugging");
+	  }
+
+	_svmm_gzmfree_ubytes (name);
+      }
+      break;
+
+    case '[':
+      {
+	jint length = 0;
+	char *p = (char *) current;
+	_svmt_u8 *name;
+
+	while (*p == '[')
+	  {
+	    p++;
+	    length++;
+	  }
+
+	if (*p == 'L')
+	  {
+	    while (*p != ';')
+	      {
+		p++;
+		length++;
+	      }
+	    length++;
+	  }
+	else
+	  {
+	    length++;
+	  }
+
+	if (_svmm_gzmalloc_ubytes (env, length + 1, name) != JNI_OK)
+	  {
+	    _svmm_fatal_error ("unsupported failure while debugging");
+	  }
+
+	p = (char *) name;
+	while (*current == '[')
+	  {
+	    *(p++) = *current++;
+	  }
+
+	if (*current == 'L')
+	  {
+	    while (*current != ';')
+	      {
+		*(p++) = *current++;
+	      }
+	    *(p++) = *current++;
+	  }
+	else
+	  {
+	    *(p++) = *current++;
+	  }
+
+	(*pindex)++;
+
+	if (_svmm_create_type
+	    (env, method_info->class_info->class_loader_info,
+	     (const char *) name, result) != JNI_OK)
+	  {
+	    _svmm_fatal_error ("unsupported failure while debugging");
+	  }
+
+	if (_svmf_link_type (env, result) != JNI_OK)
+	  {
+	    _svmm_fatal_error ("unsupported failure while debugging");
+	  }
+
+	_svmm_gzmfree_ubytes (name);
+      }
+      break;
+
+    default:
+      {
+	_svmm_fatal_error ("impossible control flow");
+      }
+    }
+
+  *pcurrent = current;
+  return result;
+}
+
+#endif /* !NDEBUG */
+
+/*
+----------------------------------------------------------------------
+_svmm_skip_type_within_descriptor
+----------------------------------------------------------------------
+*/
+
+#ifndef NDEBUG
+
+#define _svmm_skip_type_within_descriptor(current, index) \
+_svmh_skip_type_within_descriptor (&current, &index)
+
+svm_static void
+_svmh_skip_type_within_descriptor (const char **pcurrent, jint *pindex)
+{
+  const char *current = *pcurrent;
+
+  switch (*current++)
+    {
+    case 'B':
+    case 'C':
+    case 'F':
+    case 'I':
+    case 'S':
+    case 'Z':
+    case 'V':
+      {
+	(*pindex)++;
+      }
+      break;
+
+    case 'D':
+    case 'J':
+      {
+	(*pindex) += 2;
+      }
+      break;
+
+    case 'L':
+      {
+	while (*current++ != ';');
+
+	(*pindex)++;
+      }
+      break;
+
+    case '[':
+      {
+	while (*current == '[')
+	  {
+	    current++;
+	  }
+
+	if (*current++ == 'L')
+	  {
+	    while (*current++ != ';');
+	  }
+
+	(*pindex)++;
+      }
+      break;
+
+    default:
+      {
+	_svmm_fatal_error ("impossible control flow");
+      }
+    }
+
+  *pcurrent = current;
+}
+
+#endif /* !NDEBUG */
+
 /*
 ----------------------------------------------------------------------
 _svmf_print_statistics
