@@ -12,7 +12,7 @@ _svmf_verify_class
 ----------------------------------------------------------------------
 */
 
-static jint
+svm_static jint
 _svmf_verify_class (_svmt_JNIEnv *env, _svmt_class_info *class)
 {
   if (_svmf_is_set_flag (class->state, SVM_TYPE_STATE_VERIFIED))
