@@ -151,6 +151,13 @@ dummy ()
 
      m4svm_on(0)m4_dnl */
 
+  /* structured locking *[::]/
+     m4svm_off()
+
+     m4svm_property_boolean(sablevm.structured.locking, vm->enforce_structured_locking)
+
+     m4svm_on(0)m4_dnl */
+
 #if defined (_SABLEVM_NO_GC)
 
   /* copying heap parameters *[::]/
