@@ -139,7 +139,7 @@ _svmf_zip_file_open (_svmt_JNIEnv *env, const char *file_name)
   if (_svmm_gzalloc_zip_file (env, zip_file) != JNI_OK)
     goto error;
 
-  if (_svmm_gmalloc_cchars (env, strlen (file_name), zip_file->name) !=
+  if (_svmm_gmalloc_cchars (env, strlen (file_name) + 1, zip_file->name) !=
       JNI_OK)
     goto error;
   strcpy (zip_file->name, file_name);
@@ -211,6 +211,15 @@ _svmf_zip_file_open (_svmt_JNIEnv *env, const char *file_name)
 	  (zip_file->content, position, ZIP_CENTRAL_DIR_MAGIC))
 	goto error;		/* Central directory file header mismatch. */
 
+      zip_file->entries[entry].compression_method = _svmf_zip_read_value
+	(zip_file->content, position + ZIP_CENTRAL_DIR_COMPRESSION_OFFSET, 2);
+
+      zip_file->entries[entry].size_compressed = _svmf_zip_read_value
+	(zip_file->content, position + ZIP_CENTRAL_DIR_COMP_SIZE_OFFSET, 4);
+
+      zip_file->entries[entry].size_uncompressed = _svmf_zip_read_value
+	(zip_file->content, position + ZIP_CENTRAL_DIR_UNCOMP_SIZE_OFFSET, 4);
+
       field_len = _svmf_zip_read_value
 	(zip_file->content, position + ZIP_CENTRAL_DIR_FNAME_LEN_OFFSET, 2);
 
@@ -343,6 +352,7 @@ svm_static jint
     goto error;			/* Local file header mismatch. */
 
   {
+    _svmt_u32 data_position;
 #ifdef COMMENT
 /* 
  * Theorhetically we should require min_ver be <= 2.0 and bit_flags
@@ -359,19 +369,29 @@ svm_static jint
        ZIP_LOCAL_FILE_BIT_FLAG_OFFSET, 2);
 #endif
 
-    jchar compression_method = _svmf_zip_read_value
-      (zip_file->content, content_position +
-       ZIP_LOCAL_FILE_COMPRESSION_OFFSET, 2);
+/* 
+ * Information about compression method and file sizes might be available
+ * in one of two plcaes: either in the central directory or in the local
+ * file header. For files that with file size 0 in the central directory,
+ * we will try to read them here from the local directory.  If both
+ * locations contain invalid data, the sanity checks below will fail.
+ */
+    if (zip_file_entry->compression_method == 0)
+      zip_file_entry->compression_method = _svmf_zip_read_value
+	(zip_file->content, content_position +
+	 ZIP_LOCAL_FILE_COMPRESSION_OFFSET, 2);
 
-    _svmt_u32 size_compressed = _svmf_zip_read_value
-      (zip_file->content, content_position +
-       ZIP_LOCAL_FILE_COMP_SIZE_OFFSET, 4);
+    if (zip_file_entry->size_compressed == 0)
+      zip_file_entry->size_compressed = _svmf_zip_read_value
+	(zip_file->content, content_position +
+	 ZIP_LOCAL_FILE_COMP_SIZE_OFFSET, 4);
 
-    _svmt_u32 size_uncompressed = _svmf_zip_read_value
-      (zip_file->content, content_position +
-       ZIP_LOCAL_FILE_UNCOMP_SIZE_OFFSET, 4);
+    if (zip_file_entry->size_uncompressed == 0)
+      zip_file_entry->size_uncompressed = _svmf_zip_read_value
+	(zip_file->content, content_position +
+	 ZIP_LOCAL_FILE_UNCOMP_SIZE_OFFSET, 4);
 
-    _svmt_u32 data_position = content_position + ZIP_LOCAL_FILE_HEADER_SIZE +
+    data_position = content_position + ZIP_LOCAL_FILE_HEADER_SIZE +
       _svmf_zip_read_value
       (zip_file->content, content_position +
        ZIP_LOCAL_FILE_NAME_LEN_OFFSET, 2) +
@@ -380,15 +400,15 @@ svm_static jint
        ZIP_LOCAL_FILE_EXTRA_FLD_LEN_OFFSET, 2);
 
     /* Avoid segfaults in case of corrupted data in zip file. */
-    if ((data_position + size_compressed >= zip_file->file_len)
-	|| (size_compressed == 0))
+    if ((data_position + zip_file_entry->size_compressed >=
+	 zip_file->file_len) || (zip_file_entry->size_compressed == 0))
       goto error;		/* Damaged file, wrong compressed file size. */
 
     *file_content = NULL;
     *free_after_use = JNI_FALSE;
-    *file_length = size_uncompressed;
+    *file_length = zip_file_entry->size_uncompressed;
 
-    switch (compression_method)
+    switch (zip_file_entry->compression_method)
       {
       case ZIP_COMPRESSION_STORED:	/* No compression, just give out the pointer. */
 	*free_after_use = JNI_FALSE;
@@ -399,15 +419,16 @@ svm_static jint
 	  z_stream zip_stream;
 	  int res;
 
-	  if (_svmm_gmalloc_ubytes (env, size_uncompressed, *file_content) !=
-	      JNI_OK)
+	  if (_svmm_gmalloc_ubytes
+	      (env, zip_file_entry->size_uncompressed,
+	       *file_content) != JNI_OK)
 	    goto error;
 
 	  *free_after_use = JNI_TRUE;
 	  memset (&zip_stream, 0, sizeof (zip_stream));
-	  zip_stream.avail_in = size_compressed;
+	  zip_stream.avail_in = zip_file_entry->size_compressed;
 	  zip_stream.next_in = &zip_file->content[data_position];
-	  zip_stream.avail_out = size_uncompressed;
+	  zip_stream.avail_out = zip_file_entry->size_uncompressed;
 	  zip_stream.next_out = *file_content;
 	  zip_stream.data_type = Z_BINARY;
 	  zip_stream.zalloc = &_svmf_zip_alloc;
@@ -447,8 +468,8 @@ svm_static jint
 	    }
 	  inflateEnd (&zip_stream);
 
-	  if (zip_stream.total_in != size_compressed ||
-	      zip_stream.total_out != size_uncompressed)
+	  if (zip_stream.total_in != zip_file_entry->size_compressed ||
+	      zip_stream.total_out != zip_file_entry->size_uncompressed)
 	    goto error;		/* Mismatch of data length information. */
 	}
 	break;
