@@ -897,6 +897,8 @@ printFullVersion (void)
   printf ("- no garbage collection\n");
 #elif defined (_SABLEVM_COPY_GC)
   printf ("- copying garbage collection\n");
+#elif defined (_SABLEVM_GENCOPY_GC)
+  printf ("- generational copying garbage collection\n");
 #endif
 
 #ifdef _SABLEVM_TRADITIONAL_OBJECT_LAYOUT
@@ -1309,13 +1311,13 @@ main (const int argc, const char *argv[])
     jint option = 0;
 
     vm_options[option].optionString = (char *) "vfprintf";
-    vm_options[option++].extraInfo = (void *) vfprintf;
+    vm_options[option++].extraInfo = SVM_CAST_EXTENSION (void *) vfprintf;
 
     vm_options[option].optionString = (char *) "exit";
-    vm_options[option++].extraInfo = (void *) exit;
+    vm_options[option++].extraInfo = SVM_CAST_EXTENSION (void *) exit;
 
     vm_options[option].optionString = (char *) "abort";
-    vm_options[option++].extraInfo = (void *) abort;
+    vm_options[option++].extraInfo = SVM_CAST_EXTENSION (void *) abort;
 
     if (class_path != NULL)
       {
